import javax.realtime.*; 
/**
 * Use an Aysnc Interrupt that is aimed at another method, then let it
 * propagate on up the stack to its original target.
 */
public class BorrowAIE
{
  public class MyThread extends RealtimeThread
  {

    /**  This is the run method for a subclass of RealtimeThread.
     *   It creates a Timed object and runs it.
     *   This is the timed object that will call a nested method that
     *   will ultimately borrow the AIE intended for the Timed object.
     */
    public void run(){
      Timed timed = new Timed(new RelativeTime(2000,0)); 
      timed.doInterruptible(
      new Interruptible() {public void run(
        AsynchronouslyInterruptedException aie)
        throws AsynchronouslyInterruptedException{
          for(int i=0;i<20;++i){
            System.out.print("."); 
            nest(aie); 
            try{
              Thread.sleep(10); 
            }catch(Exception ie){}
          }
        }
        public void interruptAction(
        AsynchronouslyInterruptedException e){
          System.out.println("In interruptAction"); 
        }
      }
      ); 
    }
  }
  
  /**
   * This method is passed a reference to an AIE
   * that is aimed at a function up the stack.
   * It will intercept that AIE--and use it, then
   * let it propagate up the stack.
   * Note that this method does NOT throw AIE.  It can, therefore,
   * catch AIE.
   */
  public void nest(AsynchronouslyInterruptedException pAIE)
  {
    AsynchronouslyInterruptedException aie =
        new AsynchronouslyInterruptedException(); 
    
    //...do something that might get an AIE fired
    // at us.
    try{
      nestedAIE(); 
    }catch(AsynchronouslyInterruptedException cAie){
      if(pAIE.happened(false)){
        // This AIE belonged to the Timed
        // method that called us, but we've
        // grabbed it.  We asked if it happened,
        // and made it non-pending.
        System.out.println("BorrowAIE"); 
        // Now we want to return it.
        // It is not pending so it cannot
        // autopropagate, but we can throw it.
        pAIE.fire(); 
      }else if(aie.happened(false)){
        // This aie was aimed at us.
      }else{
        // It belongs to someone else.
        AsynchronouslyInterruptedException.
        propagate(); 
      }
    }
  }
  
  /**  This method throws AIE, so it is asynchronously interruptible.
   *   It does nothing but wait around to be interrupted.
   */
  public void nestedAIE() throws
      AsynchronouslyInterruptedException
  {
    for(int i=0;i<30;++i){
      try{
        Thread.sleep(10); 
      }catch(InterruptedException e){}
    }
  }
  
  /**  This is the constructor for the BorrowAIE class.
   *   It starts a realtime thread running and waits for it to end.
   */
  public BorrowAIE()
  {
    MyThread rt1 = new MyThread(); 
    rt1.start(); 
    try{
      rt1.wait(); 
    }catch(Exception e){}
  }
  
  /** This main does nothing but create an instance of the this object and
   *  get things going.
   */
  public static void main(String [] args)
  {
    new BorrowAIE(); 
  }
}
