import javax.realtime.*; 

/**  Demonstrate firing an AIE, and catching it using doInterruptible.
 */

public class One extends RealtimeThread
{
  AsynchronouslyInterruptedException aie; 

  public One(PriorityParameters pty){
    super(pty);
  }

  public One() {
    this(null);
  }
  
  /**  This is the run method for class One.
   *   It will waste time until it is interrupted, then
   *   use the doInterruptible mechanisms to handle the
   *   exception.
   */
  public void run()
  {
    One.this.aie =
        new AsynchronouslyInterruptedException(); 
    
    One.this.aie.doInterruptible(new Interruptible() {
      private volatile int n; 
      public void interruptAction(
      AsynchronouslyInterruptedException aie){
        System.out.println("Interrupted at " + n); 
      }
      
      public void run(
      AsynchronouslyInterruptedException aie)
      throws
      AsynchronouslyInterruptedException {
        for(int i=0;i<100000;++i){
          try{
            Thread.sleep(200);
          }catch(Exception e){}
          n += 1; 
        }
        System.out.println("Something's wrong."); 
      }
    }
    ); 
  }
  
  /**  Start one realtime thread to receive an AIE, target.
   *   Start one realtime thread to fire the AIE, killer.
   *   Wait for them both to finish.
   */
  public static void main(String [] args)
  {
    final One target  = new One(new PriorityParameters(
        PriorityScheduler.instance().getNormPriority())); 
    PriorityParameters killerPty = new PriorityParameters(
        PriorityScheduler.instance().getNormPriority() + 1);
    RealtimeThread killer = new RealtimeThread(killerPty) {
      public void run(){
        try{
          Thread.sleep(1000); 
          if(target.aie.fire()) 
            System.out.println("Fire returned true"); 
          else 
            System.out.println("Fire returned false"); 
        }catch(InterruptedException e){}
      }
    };
    killer.start(); 
    target.start(); 
    try{
      target.join(); 
      killer.join(); 
    }catch(InterruptedException e){}
  }
}
