import javax.realtime.*; 

/**  Demonstrate Intercepting AIE entended for another method, and
 *   not letting the other method see it.
 */
public class StealAIE
{
  public class MyThread extends RealtimeThread
  {
    /**  Use Timed to set up interruptible code and an AIE that
     *   will be fired at it in 2 seconds.
     */
    public void run(){
      Timed timed = new Timed(new RelativeTime(2000,0)); 
      timed.doInterruptible(
      new Interruptible() {public void run(
        AsynchronouslyInterruptedException aie)
        throws AsynchronouslyInterruptedException{
          for(int i=0;i<20;++i){
            System.out.print("."); 
            nest(aie); 
            try{
              Thread.sleep(10); 
            }catch(Exception ie){}
          }
        }
        public void interruptAction(
        AsynchronouslyInterruptedException e){
          System.out.println("In interruptAction"); 
        }
      }
      ); 
    }
  }
  
  /**
   * This method is passed a reference to an AIE
   * that is aimed at a function up the stack.
   * It will intercept that AIE--not letting it
   * propagate up the stack.
   */
  public void nest(
      AsynchronouslyInterruptedException pAIE)
  {
    AsynchronouslyInterruptedException aie =
        new AsynchronouslyInterruptedException(); 
    
    //...do something that might get an AIE fired
    // at us.
    try{
      nestedAIE(); 
    }catch(AsynchronouslyInterruptedException cAie){
      if(pAIE.happened(false)){
        // This AIE belonged to the Timed
        // method that called us, but we've
        // grabbed it.  We asked if it happened,
        // and made it non-pending.
        // We could rethrow it with throw pAIE.
        // but we're just going to steal it.
        System.out.println("Stealing"); 
      }else if(aie.happened(false)){
        // This aie was aimed at us.
      }else{
        // It belongs to someone else.
        AsynchronouslyInterruptedException.
            propagate(); 
      }
    }
  }
  
  public void nestedAIE() throws
      AsynchronouslyInterruptedException
  {
    for(int i=0;i<30;++i){
      try{
        Thread.sleep(10); 
      }catch(InterruptedException e){}
    }
  }
  
  public StealAIE()
  {
    MyThread rt1 = new MyThread(); 
    rt1.start(); 
    try{
      rt1.wait(); 
    }catch(Exception e){}
  }
  
  public static void main(String [] args)
  {
    new StealAIE(); 
  }
}
