import javax.realtime.*; 

/**
 * Properly catch a generic AIE.
 */

public class Two extends RealtimeThread
{
  static AsynchronouslyInterruptedException generic =
  AsynchronouslyInterruptedException.getGeneric();
  private int n=0; 

  public Two(PriorityParameters pty){
    super(pty);
  }

  public Two() {
    this(null);
  }
  
  /**  Call doIt to waste time while being asynchronously interruptible,
   *   When doIt throws AIE, catch it.  If it is the "generic" AIE, handle it
   *   otherwise let it propagate.
   */
  public void run()
  {
    try{
      doIt(); 
    }catch(AsynchronouslyInterruptedException aie){
      // If it is generic,
      // make this aie non-pending
      if(generic.happened(false)){
        System.out.println("Run caught it. n = " + n); 
      }else{
        System.out.println(
        "Not generic AIE, auto-propagated"); 
        // let the exception fall out and be re-thrown
      }
    }
  }
  
  /**	Waste time until we are interrupted.
   */
  private void doIt() throws AsynchronouslyInterruptedException
  {
    while(true){
      n += 1; 
      try{
        sleep(200);
      }catch(InterruptedException e){}
    }
  }
 

  /** The main for this demonstration.  Start rt1 to receive and AIE,
   *  and rt2 to interrupt it.
   */ 
  public static void main(String [] args)
  {
    PriorityParameters normPty = new PriorityParameters(
        PriorityScheduler.instance().getNormPriority());
    PriorityParameters killerPty = new PriorityParameters(
        PriorityScheduler.instance().getNormPriority() + 1);
    final Two rt1 = new Two(killerPty); 
    RealtimeThread rt2 = new RealtimeThread(normPty) {
      public void run(){
        try{
          Thread.sleep(1000); //  Interrupt the target thread after 1 second.
          rt1.interrupt(); 
        }catch(InterruptedException e){}
      }
    };
    rt2.start(); 
    rt1.start(); 
    try{
      rt1.join(); 
      rt2.join(); 
    }catch(InterruptedException e){}
  }
}
