import javax.realtime.*; 
/** Run two no-heap threads that share some common data
 *  that was allocated in a scope in a shared part of the
 *  threads's scope stacks.
 */
public class Sharing extends RealtimeThread
{
  /**
   * Something to share
   */
  static class Sharable
  {
    private int value=0; 
    synchronized void modify(int x){
      value += x;
    }

    int getValue(){ 
      return value;
    }
  }
  
  /**
   * A little Noheap thread
   */
  static class NhRTT extends NoHeapRealtimeThread
  {
    Sharable foo; 
    int adder; 
    /**
     * Constructor for NhRTT
     */
    public NhRTT(SchedulingParameters sched,
        MemoryArea mem,
        int adder,
        Sharable foo)
    {
      super(sched, mem); 
      this.foo = foo; 
      this.adder = adder; 
    }

    public void run(){
      for(int i=0;i<20;++i){
        foo.modify(adder); 
        try{
          sleep(500); 
        }catch(InterruptedException e){}
      }
    }
  }
  
  /**
   * Run method for a RealtimeThread
   */
  public void run()
  {
    // Create the memory area the noheap thread will use
    VTMemory mem0 = new VTMemory(8*1024, 8*1024); 
    final VTMemory mem1 = new VTMemory(8*1024, 8*1024); 
    final VTMemory mem2 = new VTMemory(8*1024, 8*1024); 
    
    mem0.enter(new Runnable() {public void run(){
        // Make the things the threads will need
        final Sharable foo = new Sharable(); 
        NhRTT nhrtt1 = new NhRTT(null, mem1, 2, foo); 
        NhRTT nhrtt2 = new NhRTT(null, mem1, -1, foo); 
        
        // The threads don't need the monitor,
        // but this is a good place to make it.
        Runnable monitor = new Runnable() {
          public void run(){
            System.out.print(
                foo.getValue() + " "); 
            try{
              Thread.currentThread().sleep(100); 
            }catch(InterruptedException e){}; 
          }
        };
        // Start the threads
        nhrtt1.start(); 
        try{
          // Put a little delay between the threads
          sleep(200); 
        }catch(InterruptedException e){}
        nhrtt2.start(); 
        
        // Watch them run for a while.
        for(int i=0; i<100; ++i) mem2.enter(monitor); 
        
        try{
          nhrtt1.join(); 
          nhrtt2.join(); 
        }catch(InterruptedException e){}
      }
    }
    ); 
  }
  
  public static void main(String [] args)
  {
    try{
        Sharing rtt = (Sharing)ImmortalMemory.instance().newInstance(Sharing.class);
        rtt.start(); 
        try{
          rtt.join(); 
        }catch(InterruptedException e){}
    }  catch (IllegalAccessException e2){
        System.out.println(e2);
        e2.printStackTrace();
    }   catch (InstantiationException e2){
        System.out.println(e2);
        e2.printStackTrace();

    }
  }

}
