import javax.realtime.*; 
/** Demonstrate use of the raw memory access class.
 *  This class pretends to begin to implement an Ethernet driver.
 *  So far all it has is the beginning of the init routine.
 */
public class Driver
{
  private final long CTRLREG = 20; 
  private final long STATREG = 24; 
  private final int MASK_INT_FIELDS = 0xf0ffffff; 
  private final int MY_NOINT_FIELD = 0x05000000; 
  private final int INIT_CTRL = 0x104f3107; 
  private RawMemoryAccess eDvc; 

  /** The constructor for the "driver."
   *  This method gets a reference for the control registers
   *  for the right kind of Ethernet device.
   */
  public Driver()
  {
    String [] type = new String[4]; 
    type[0] = new String("device"); 
    type[1] = new String("ethernet"); 
    type[2] = new String("Intel"); 
    type[4] = new String("no cache"); 
    try{
      eDvc = new RawMemoryAccess(type, 512); 
    }catch(SizeOutOfBoundsException e){
      // Programming error
      throw new RuntimeException("Size out of bounds ln Driver"); 
    }catch(OffsetOutOfBoundsException e){
      // This should not happen
    }catch(MemoryTypeConflictException e){
      throw new RuntimeException("No compatible device found"); 
    }catch(UnsupportedPhysicalMemoryException e){
      throw new RuntimeException("No compatible device found"); 
    }
  }
  
  /**  Pretend to initialize the device by accessing its control
   *   and status registers.
   */
  public void init()
  {
    try{
      // Hang onto the old value of the control
      // register
      int ctrlReg = eDvc.getInt(CTRLREG); 
      // Set an initial state
      eDvc.setInt(CTRLREG, 0); 
      // Turn off interrupts
      eDvc.setInt(CTRLREG, (ctrlReg & MASK_INT_FIELDS) | MY_NOINT_FIELD); 
      // Wait for device to respond
      while(eDvc.getInt(STATREG)!=0) ;
      // Set it running
      eDvc.setInt(CTRLREG, INIT_CTRL); 
    }catch(SizeOutOfBoundsException e){
      throw new RuntimeException(
      "Size error in init"); 
    }catch(OffsetOutOfBoundsException e){
      throw new RuntimeException(
      "Offset error in init"); 
    }
  }
}

