import javax.realtime.*; 
/** Demonstrate a few operations through raw memory access. */
public class Test2
{
  private final int BUFFER_SIZE = 512 * 1024; 
  private final int BUFFER_DIM = BUFFER_SIZE/2; 
  // Five bits per color
  private final short BLUE_VALUE = 0x001f; 
  RawMemoryAccess frameBuffer; 

  /** The constructor gets access to 512k of frame buffer memory */
  public Test2()
  {
    try{
      frameBuffer = new RawMemoryAccess("frame buffer", BUFFER_SIZE); 
    }catch(SecurityException e){
      System.err.println("Not allowed to access frame buffer"); 
      return; 
    }catch(OffsetOutOfBoundsException e){
      System.err.println("Bad address"); 
      return; 
    }catch(SizeOutOfBoundsException e){
      System.err.println("Frame buffer is less than 512k"); 
      return; 
    }catch(UnsupportedPhysicalMemoryException e){
      System.err.println("Frame buffer unknown"); 
      return; 
    }catch(MemoryTypeConflictException e){
      // Since we only asked for one type qualifier
      // and we didn't specify a base address, we can not
      // actually get this exception.
      System.err.println("Incompatible types"); 
      return; 
    }
  }
  
  /** Use three different ways to clear the frame buffer */
  public void clear()
  {
    try{
      // One way
      for(int offset=0;offset<BUFFER_SIZE;offset+=2) 
        frameBuffer.setShort(offset, BLUE_VALUE); 
      // Another way
      short [] filler = new short[BUFFER_DIM]; 
      for(int i=0;i<BUFFER_DIM;++i) 
        filler[i] = BLUE_VALUE; 
      frameBuffer.setShorts(0, filler, 0, BUFFER_DIM); 
      // Or yet another way
      for(int i=0;i<BUFFER_DIM;++i) 
        frameBuffer.setShorts(i, filler, 0, 1); 
    }catch(SizeOutOfBoundsException e){
      // Program bug
    }catch(OffsetOutOfBoundsException e){
      // Program bug
    }
  }
  
  /** Swap the bytes in a two-byte frame buffer entry.
   *  @param n Index into the frame buffer treated as an array
   *           of shorts.
   */
  public void swap(int n) throws OffsetOutOfBoundsException
  {
    try{
      short s1 = frameBuffer.getShort(n*2); 
      byte [] bytes = new byte[2]; 
      
      frameBuffer.getBytes(n*2, bytes, 0, 2); 
      byte hold = bytes[0]; 
      bytes[0] = bytes[1]; 
      bytes[1] = hold; 
      frameBuffer.setBytes(n*2, bytes, 0, 2); 
      short s2 = frameBuffer.getShort(n*2); 
    }catch(SizeOutOfBoundsException e){
      throw new OffsetOutOfBoundsException(); 
    }catch(OffsetOutOfBoundsException e){
      throw e; 
    }
  }
}
