import javax.realtime.*; 

/**
 * Demonstrate an aggressive miss handler
 */
public class AggressiveMissHdlr
{
  /**
   * Define the AEH for overruns
   */
  public static class MissHdlr extends
  AsyncEventHandler
  {
    AsynchronouslyInterruptedException aie; 
    RealtimeThread rtt; 
    
    public void setAIE(AsynchronouslyInterruptedException aie){
      this.aie = aie; 
    }
    
    public void setThread(RealtimeThread rtt){
      this.rtt = rtt; 
    }
    
    MissHdlr(){
      super(
          new PriorityParameters(
              PriorityScheduler.MIN_PRIORITY+11),
              null, null, null, null, null); 
    }
    
    public void handleAsyncEvent(){
      // First manage the consequences of missing
      // the deadline
      // ...
      
      System.out.println("Zapping missed computation"); 
      // Let the target thread go in case it is
      // stuck in waitForNextPeriod()
      rtt.schedulePeriodic(); 
      if(!aie.fire()) //	Throw an AIE into the thread
        System.out.println("Fire returned false"); 
    }
  }
  
  /**
   * This class is the executable body of the periodic
   * thread phrased as an Interruptible so it can be
   * used with doInterruptible.
   */
  public static class MyInterruptible implements Interruptible
  {
    public void run(AsynchronouslyInterruptedException e)
        throws AsynchronouslyInterruptedException {
      int bound=0; 
      double f; 
      int ct = 0; 
      RealtimeThread rtt =
          RealtimeThread.currentRealtimeThread(); 
      
      while(true){
        do{
          if(++ct>20) return; 
          for(f=0.0;f<bound;f+=1.0);
          bound += 100000; 
          System.out.println("Ding! " + bound); 
          try{
            Thread.sleep(1); 
          }catch(Exception ie){}
        }while(rtt.waitForNextPeriod());
        // Miss never gets here because
        // it causes an AIE
        System.out.println("Shouldn't ever get here"); 
      }
    }
    
    public void interruptAction(
    AsynchronouslyInterruptedException e){
      // Recover from missing the deadline
      System.out.println("Recovering."); 
    }
  }
  
  /**
   * Define a RT thread that stops quickly in case
   * of an AIE.
   */
  public static class PeriodicThread extends RealtimeThread
  {
    volatile MissHdlr missHdlr; 
    
    /**
     * Constructor
     */
    public PeriodicThread(SchedulingParameters sched, 
        ReleaseParameters release){
      super(sched, release); 
      missHdlr = (MissHdlr)release.getDeadlineMissHandler(); 
    }
    
    /**
     * Run method for the PeriodicThread
     */
    public void run() {
      AsynchronouslyInterruptedException aie =
          new AsynchronouslyInterruptedException(); 
      
      // Tell the miss handler about our AIE
      missHdlr.setAIE(aie);
      aie.doInterruptible(new MyInterruptible()); 
    }
  }
  
  public static void main(String [] args)
  {
    // Build parameters for construction
    // of RT thread
    MissHdlr missHdlr = new MissHdlr(); 
    ReleaseParameters release = new PeriodicParameters(
        new RelativeTime(), // Start at .start()
        new RelativeTime(1000, 0), // 1 second period
        null, // cost
        new RelativeTime(100, 0),// deadline=period/10
        null, // no overrun handler
        missHdlr); // miss handler
    SchedulingParameters scheduling = new PriorityParameters(
        PriorityScheduler.instance().getMinPriority()+10); 
    
    RealtimeThread rt= new PeriodicThread(scheduling, release); 
    // Give the miss handler a reference to
    // the thread it is managing.
    missHdlr.setThread(rt);
    
    rt.start(); // Start the periodic thread
    try{
      rt.join(); // Wait for the thread to end
    }catch(Exception e){}; 
  }
}
