import javax.realtime.*; 

/**
 * Demonstrate a Passive miss handler 
 */
public class PassiveMissHdlr {
    private static final int THREAD_PRIORITY = 10;
    private static final int HANDLER_PRIORITY = 11;
    /**
     * Define the passive AEH for Misses
     */
    public static class MissHdlr extends AsyncEventHandler {
        PeriodicThread th; 
        private int failureCt;
        private static final int FAILURE_ALLOWANCE = 5;

        public void setThread(PeriodicThread th) {
            this.th = th; 
        }

        MissHdlr() {
            super(new PriorityParameters(
                  PriorityScheduler.instance().getMinPriority() + HANDLER_PRIORITY),
                  null, null, null, null, null); 
        }

        public void handleAsyncEvent() {
            // First interact with whatever is bothered
            // by us missing the deadline,

            // then deal with the thread that missed
            if (failureCt++ < FAILURE_ALLOWANCE) {
                System.out.println("Recovering from a miss");
            } else {
                System.out.println("Too many misses. Stopping thread.");
                th.stopNow = true;
            }
            th.schedulePeriodic(); // Let the thread continue
        }
    }

    /**
     * Define a simple periodic RT thread
     */
    public static class PeriodicThread extends RealtimeThread {
        volatile double f; 
        volatile boolean stopNow = false;

        public PeriodicThread(SchedulingParameters sched, 
                              ReleaseParameters release) {
            super(sched, release); 
        }

        /** A run method that causes lots of deadline misses. */
        public void run() {
            int bound = 0; 

            do {
                if (stopNow) {
                    return;
                }
                for (f=0.0;f<bound;f+=1.0) ; // Use some time
                bound += 100000; // Use more next time
                System.out.println("Ding! " + bound); 
            }while (waitForNextPeriod());    // Never returns false.

        }
    }

    public static void main(String [] args) {
        // Build parameters for construction
        // of RT thread
        MissHdlr missHdlr = new MissHdlr(); 
        ReleaseParameters release = new PeriodicParameters(
                                          new RelativeTime(), // Start at .start()
                                          new RelativeTime(1000, 0), // 1 second period
                                          null, // cost
                                          new RelativeTime(500,0),// deadline=period/2
                                          null, // no overrun handler
                                          missHdlr); // miss handler
        SchedulingParameters scheduling = new PriorityParameters(
                                                                PriorityScheduler.instance().getMinPriority() + THREAD_PRIORITY); 
        PeriodicThread rt= new PeriodicThread(scheduling, release); 
        // Give the miss handler a reference to
        // the thread it is managing.
        missHdlr.setThread(rt);

        rt.start(); // Start the periodic thread
        try {
            rt.join(); // Wait for the thread to end
        } catch (Exception e) {
        }; 
    }
}
