import javax.realtime.*;

public class InstanceRef1 {

  public class Test implements Runnable {

    public void run(){
      //  The first example will throw an exception when it tries to
      //  store into n

      //  make the mems final so they can be seen from in the closures.
      final LTMemory mem1 = new LTMemory(8 * 1024, 8 * 1024);
      final LTMemory mem2 = new LTMemory(8 * 1024, 8 * 1024);
      IdMemArea.addNew(mem1, "inner scope 1");
      IdMemArea.addNew(mem2, "inner scope 2");

      //
      //  First test: Try to store a reference to an object created in
      //  an inner scope to a field in the next scope out.
      //  (This is a bit tricky since the reference doesn't _look_
      //  like it's in a different scope.)
      //
      try{
        mem1.enter(new Runnable() {
          Integer n;
          public void run(){
            n = new Integer(1);
          }
        });
      }catch (Throwable e){
        System.err.println("Expected exception in first test");
      }

      //
      // Second test:  What if we create a Runnable first?
      //
      try{
        Runnable scope2 = new Runnable(){
          Integer n;
          public void run(){
            n = new Integer(1);
          }
        };
        mem1.enter(scope2);
      }catch (Throwable e){
        System.err.println("Expected exception in second test");
      }

      //
      //  Third test: What if we use a named inner class for the
      //  Runnable, and have it create the object in the scope
      //  where it wants to save the reference?
      //
      try{
        //  Named inner class
        class XScope implements Runnable {
          StringBuffer s;
          public void run(){
            RealtimeThread thisThread = 
              RealtimeThread.currentRealtimeThread();
            int currentIdx = RealtimeThread.getMemoryAreaStackDepth() - 1;
            int idx = currentIdx - 1;
            MemoryArea outerScope = 
                (MemoryArea)RealtimeThread.getOuterMemoryArea(idx);
            try {
              s = (StringBuffer)outerScope.newInstance(StringBuffer.class);
            } catch (InstantiationException instance){
            } catch (IllegalAccessException access) {
            }
            s.append("Hello World");
          }
        };

        System.out.println("Outside scope: " + IdMemArea.current());

        XScope scope3 = new XScope();
        mem1.enter(scope3);
        System.out.println(scope3.s.toString());

      }catch (Throwable e){
        System.err.println("Throw in third test");
        e.printStackTrace();
      }
    }
  }
  
  public void go(){
    RealtimeThread rt = new RealtimeThread(null, null, null,
      HeapMemory.instance(),
      null, new Test()
    );
    rt.start();
    try{
      rt.join();
    } catch(InterruptedException e){}
  }

  public static void main(String [] args){
    InstanceRef1 t = new InstanceRef1();
    t.go();
  }
}
