import javax.realtime.*; 
/**
 * Demonstrate illegal nested scopes.
 * (Scoped cycle exception.)
 * This class causes a scoped cycle exception, in a heap scope.
 * This lets the exception propagate up the stack without generating
 * a boundary exception.
 */
// Nest3 solves the same problem (scope boundary exception) by
// catching the exception before it tries to get out of the scope.

public class Nest2 extends RealtimeThread
{
  String [] args; 
  int [] intArgs; 
  LTMemory mem; 
  
  public Nest2(String [] args)
  {
    this.args = args; 
    intArgs = new int[args.length]; 
  }
  
  public void run()
  {
    for(int i=0;i<this.args.length;++i){
      mem = new LTMemory(8*1024, 8*1024); 
      final int j = i; 
      mem.enter(new Runnable() {
        public void run() {
          // ... compute some
          System.out.println("Entered mem");
          HeapMemory.instance().enter(new Runnable() {
            public void run(){
              System.out.println("Entered Heap");
              // Now we're back on the heap
              // Now try to nest again
              Runnable inner = new Nest2_2();
              mem.enter(inner);
                // This is illegal
                // mem memory's parent is "primordial".
                // If we successfully entered it here it would
                // also have itself as a parent. 
                // We cannot enter it with  a scoped memory 
                // area on the stack...
                // even mem :-)
              System.out.println("Left mem");
            }
          }
          ); 
          System.out.println("Left heap");
        }
      }
      ); 
      System.out.println("Left mem");
      System.out.println(intArgs[i]); 
    }
  }
  
  static public void main(String [] args)
  {
    RealtimeThread rt = new Nest2(args); 
    rt.start(); 
    try{
      rt.join(); 
    }catch(Exception e){}; 
  }
}
