/**
 *  Demonstrate allocating data in an outer scope.
 *  Start real-time thread in heap, have it start another rt thread
 *  in heap, then have that thread enter a scope.  While in that
 *  scope it creates and stores objects in the outer scope (in this
 *  case that is heap.)
 */
import javax.realtime.*;
import java.math.*;
import java.util.*;

public class ReachOut implements Runnable {

  char [] toPrint; 
  LTMemory mem=null;

  public ReachOut(){
    this.mem = new LTMemory(1024*4, 1024*4);
    IdMemArea.addNew(this.mem, "Inner LTMemory");
  }

  /** The run method for the second RT thread created. 
   *  This is the thread that exercises enter and creating
   *  an object in an outer memory area.
   */
  public void run() {
    System.out.println("In ReachOut.run. Scope of this.toPrint: " +
      IdMemArea.id(toPrint));
    System.out.println("In ReachOut.run. Scope of this.mem: " +
      IdMemArea.id(this.mem));
    System.out.println("In ReachOut.run. Scope of this: " +
      IdMemArea.id(this));
    this.mem.enter(new Runnable() {public void run() {
      int i;
      char c [] = null;
      int currentIdx = RealtimeThread.getMemoryAreaStackDepth() - 1;
      int idx = currentIdx - 1; //  One nesting level out.
      MemoryArea mem2 = RealtimeThread.getOuterMemoryArea(idx);
      System.out.println("In ReachOut.enter.run. Scope of mem: " +
        IdMemArea.id(mem));
      System.out.println("In ReachOut.enter.run. Scope of mem2: " +
        IdMemArea.id(mem2));
      System.out.println("Current scope: " +
        IdMemArea.current());
      System.out.println("Scope stack: " +
        IdMemArea.scopeStack());
      
      try {
        c = (char [])mem2.newArray(char.class, 20);
        System.out.println(
            "In ReachOut.enter.run. Scope of c[] after newArray: " +
            IdMemArea.id(c));
      } catch (Throwable e){
        e.printStackTrace();
        System.exit(1);
      }

      try {
        BigInteger n;
        String s; 
        char c2 [] = null;
        
        n = new BigInteger(60, new Random());
        s = n.toString();
        c2 = s.toCharArray();

        for(i = 0; i < c.length; ++i)
          c[i] = ' ';
        try{
          for(i = 0; i < c2.length; ++i)
            c[i] = c2[i];
        } catch (Throwable e){
          System.out.println("Throw in copy loop");
          e.printStackTrace();
          System.exit(1);
        }
        try {
          toPrint = c;
        } catch (Throwable e){
          System.out.println("Throw in assignment to toPrint");
          e.printStackTrace();
          System.exit(1);
        }
      } catch (Throwable e){
        System.out.println("Outer catch block");
        e.printStackTrace();
      }
    }});
    if(toPrint != null)
      System.out.println(toPrint);
    else
      System.out.println("Null to Print");
  }

  public static void main(String [] args){
    //  Dummy thread to get let us allocate a ReachOut object
    //  in a scope.
    RealtimeThread rt= new RealtimeThread(
      null, null, null,
      HeapMemory.instance(),
      null, 
      new Runnable() { public void run() {
        System.out.println("In dummy thread");
        Runnable logic=null;
        try {
          logic = (Runnable) ImmortalMemory.instance().
            newInstance(ReachOut.class);
        } catch (IllegalAccessException access){
          access.printStackTrace();
          System.exit(1);
        } catch (InstantiationException instant){
          instant.printStackTrace();
          System.exit(1);
        }
        System.out.println("Mem area of logic: " + IdMemArea.id(logic));
        System.out.println("Current mem type: " + IdMemArea.current());
        //  The new thread object is created in heap, and 
        //  its initial memory area is also heap.
        RealtimeThread rt2 = new RealtimeThread(
          null, null, null,
          RealtimeThread.currentRealtimeThread().getMemoryArea(),
          null, 
          logic);
        rt2.start();
        try {
          rt2.join();
        } catch(Exception e){};
      }});
    rt.start();
    try {
      rt.join();
    } catch(Exception e){
      System.out.println("Exception returned from join");
      e.printStackTrace();
    }
  }
}
