/**
 *  Demonstrate allocating data in an outer scope
 *  For this demonstration do not use the thread scope, just a
 *  stack of nested scopes.
 */
import javax.realtime.*;
import java.math.*;
import java.util.*;

public class ReachOut2 implements Runnable {

  LTMemory mem1=null;
  LTMemory mem2=null;
  LTMemory mem3=null;

  public ReachOut2(){
    this.mem1 = new LTMemory(1024*4, 1024*4);
    this.mem2 = new LTMemory(1024*4, 1024*4);
    IdMemArea.addNew(this.mem1, "Inner LTMemory 1");
    IdMemArea.addNew(this.mem2, "Inner LTMemory 2");
  }

  public void run() {
    //  We should be on the heap here
    System.out.println("In ReachOut2.run. Scope of this.mem1: " +
      IdMemArea.id(this.mem1));
    System.out.println("In ReachOut2.run. Scope of this: " +
      IdMemArea.id(this));
    //  First enter
    this.mem1.enter(new Runnable() {
      char [] toPrint; 
      public void run() {
      //  Nested enter()
      ReachOut2.this.mem2.enter(new Runnable() {public void run() {
        int i;
        char c [] = null;
        BigInteger n;
        String s; 
        char c2 [] = null;
        int currentIdx = RealtimeThread.getMemoryAreaStackDepth() - 1;
        int idx = currentIdx -1;
        MemoryArea outerMem = RealtimeThread.getOuterMemoryArea(idx);
        System.out.println("In ReachOut2.enter.run2. Scope of mem1: " +
          IdMemArea.id(mem1));
        System.out.println("In ReachOut2.enter.run2. Scope of outerMem: " +
          IdMemArea.id(outerMem));
        System.out.println("In ReachOut2.run2. Scope of this.toPrint: " +
          IdMemArea.id(toPrint));
        System.out.println("Current scope: " + IdMemArea.current());
        
        try {
          c = (char [])outerMem.newArray(char.class, 20);
          System.out.println(
            "In ReachOut2.enter.run2. Scope of c[] after newArray: " +
            IdMemArea.id(c));
        } catch (Throwable e){
          e.printStackTrace();
          System.exit(1);
        }

        try {
          n = new BigInteger(60, new Random());
          s = n.toString();
          c2 = s.toCharArray();

          for(i = 0; i < c.length; ++i)
            c[i] = ' ';
          for(i = 0; i < c2.length; ++i)
            c[i] = c2[i];
          try {
            toPrint = c;
          } catch (Throwable e){
            System.out.println("Exception from assignment to toPrint... as expected");
            System.out.println("C is in " + 
              IdMemArea.id(c) +
              " and toPrint is in " +
              IdMemArea.id(toPrint));
          } finally {
            System.out.println(
                "There should have been a throw in assignment to toPrint");
            System.out.println("toPrint is in Heap and c is in a scope.");
          }
        } catch (Throwable e){
          System.out.println("Outer catch block");
          e.printStackTrace();
        }
      }});
      if(toPrint != null)
        System.out.println(toPrint);
      else
        System.out.println("Null to Print");
    }});
  }

  public static void main(String [] args){
    RealtimeThread rt= new RealtimeThread(
      null, null, null, HeapMemory.instance(), 
      null, (Runnable)(new ReachOut2()));
    rt.start();
    try {
      rt.join();
    } catch(Exception e){};
  }
}
