import javax.realtime.*; 
/**
 * Demonstrate the use of a portal shared between a parent thread
 * and a thread it creates.
 */

public class UsePortal extends RealtimeThread
{
  /**
   * The class that will be attached to the
   * getPortal method in a VTMemory area.
   */
  public class Port
  {
    char [] s; 
    int counter; 
    boolean stopper; 
  }
  
  /**
   * Two RT threads will use this class as
   * the logic for a memory area enter method.
   * They differ in the characters they
   * store into portal.s, and in the period
   * they use.
   */
  class ThAction implements Runnable
  {
    private Port portal; 
    private char [] tag; 
    
    /**
     * Constructor for ThAction inner class
     */
    ThAction(char [] tag){
      int idx = RealtimeThread.currentRealtimeThread().
          getMemoryAreaStackDepth() - 1; 
      portal = (Port)((VTMemory)getOuterMemoryArea(idx)).
          getPortal(); 
      this.tag = tag; 
    }
    
    /**
     * Run method for ThAction.
     */
    public void run() {
      try{
        synchronized(portal){
          portal.s[0] = tag[0]; 
          portal.s[1] = tag[1]; 
          portal.counter++; 
        }
      }catch(Throwable e){
        System.out.println("Throw in thread action" + e); 
        e.printStackTrace(); 
      }
    }
  }
  
  /**
   * Class used for the two threads that share the
   * shared memory area with the master thread.
   */
  public class TH extends RealtimeThread
  {
    private char [] tag; 
    
    /**
     * Constructor for TH
     */
    public TH(char tag1, char tag2, long period){
      super(new PriorityParameters(((PriorityParameters)RealtimeThread.currentRealtimeThread().getSchedulingParameters()).getPriority()),
          // Make this a periodic thread
          new PeriodicParameters(new RelativeTime(0,0),
              new RelativeTime(period, 0),
              null, null, null, null)
          ); 
      tag = new char[2]; 
      tag[0] = tag1; 
      tag[1] = tag2; 
    }
    
    /**
     * Run method for TH.
     */
    public void run(){
      ThAction action = new ThAction(this.tag); 
      
      VTMemory mem = new VTMemory(4096, 4096); 
      while(!action.portal.stopper){
        mem.enter(action); 
        // Wait for next period
        while(!waitForNextPeriod()) System.out.print("*");
      }
    }
  }
  
  /**
   * Run method for the UsePortal class
   */
  public void run()
  {
    (new VTMemory(20*1024, 20*1024)).enter(new Runnable(){
      public void run(){
        final VTMemory outerMem = (VTMemory)getCurrentMemoryArea(); 
        final Port portal = new Port(); 
        outerMem.setPortal(portal); 
        portal.s = new char[2]; 
        TH rt1 = new TH('t', '1', 500); 
        TH rt2 = new TH('T', '2', 300); 
        rt1.start(); 
        rt2.start(); 
        
        VTMemory mem2 = new VTMemory(20*1024, 20*1024, new Runnable() {
          public void run() {
            try{
              Thread.sleep(400); 
            }catch(InterruptedException e){
              // Ignore these exceptions
            }
            synchronized(portal){
              String s = new String(portal.s); 
              
              System.out.println(s + " = " + portal.counter +
                  " outer mem free = " + outerMem.memoryRemaining()); 
            }
          }
        }
        ); 
        for(int i = 0;i < 15; ++i) 
          mem2.enter(); 
        
        portal.stopper = true; 
        try{
          rt1.join(); 
          rt2.join(); 
        }catch(InterruptedException e){}
      }
    }
    ); 
  }
  
  public static void main(String [] args)
  {
    try{
      UsePortal rt = new UsePortal(); // make a rt thread
      rt.start(); 
      try{
        rt.join(); 
      }catch(InterruptedException e){
        // Ignore the exception
      }
      System.exit(0); 
    }catch(Throwable e){
      System.out.println("Throw in main"); 
      e.printStackTrace(); 
    }
  }
}
